import CodeMirror from 'codemirror'

import 'codemirror/addon/mode/overlay'
import 'codemirror/mode/htmlmixed/htmlmixed'
import 'codemirror/lib/codemirror.css'
import 'codemirror/mode/css/css.js'
import 'codemirror/addon/edit/closetag.js'
import 'codemirror/addon/selection/active-line.js'

import './nunjucks.css'
const regexVariable = /^{{\s*([^ }]+)\s*[^}]*\s*}}/
const regexTag = /^{%\s*([^ }]+)\s*[^%]*\s*%}/
const regexComment = /^{#\s*[^#]+\s*#}/
let ticker = 1

CodeMirror.defineMode("nunjucks:inner", function (config, parserConfig) {

  return {
    startState: function startState() {
      return { inRaw: false }
    },
    token: function token(stream, state) {
      var m = void 0
      // This makes sure that adjacent tags still have unique types
      ticker *= -1
      m = stream.match(regexTag, true)
      if (m) {
        var name = m[1]
        if (state.inRaw && name === 'endraw') {
          state.inRaw = false
        } else if (!state.inRaw && name === 'raw') {
          state.inRaw = true
        } else if (state.inRaw) {
          // Inside raw tag so do nothing
          return null
        }
        return 'nunjucks-tag ' + ticker
      }
      if (!state.inRaw) {
        m = stream.match(regexVariable, true)
        if (m) {
          return 'nunjucks-variable ' + ticker
        }
      }
      if (!state.inRaw) {
        m = stream.match(regexComment, true)
        if (m) {
          return 'nunjucks-comment ' + ticker
        }
      }
      while (stream.next() != null) {
        if (stream.match(regexVariable, false)) break
        if (stream.match(regexTag, false)) break
        if (stream.match(regexComment, false)) break
      }
      return null
    }
  }
})
CodeMirror.defineMode("nunjucks", function (config) {
  var htmlBase = CodeMirror.getMode(config, "text/html")
  var nunjucksInner = CodeMirror.getMode(config, "nunjucks:inner")
  return CodeMirror.overlayMode(htmlBase, nunjucksInner)
})
CodeMirror.defineMIME("application/x-nunjucks-template", "nunjucks")

